/*
 * Copyright (c) 2015-2019,2021-2023 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Fri Sep  8 2023
 */

#ifndef H_JOYSTICK_CODELS
#define H_JOYSTICK_CODELS

#include <sys/stat.h>

#include <aio.h>
#include <errno.h>
#include <inttypes.h>
#include <stdbool.h>

struct joystick_log_s {
  int fd;
  struct aiocb req;
  char buffer[4096];
  bool pending, skipped;
  uint32_t decimation;
  size_t missed, total;

# define joy_log_header_fmt                                             \
  "ts en vx vy vz wx wy wz ax ay az aax aay aaz"
};


static inline genom_event
joy_e_sys_error(const char *s, genom_context self)
{
  joystick_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return joystick_e_sys(&d, self);
}

void	joystick_mkstateflags(int button, int pressed,
                const joystick_mappings_s *mappings,
                const or_joystick_state *state, joystick_input_s *input);
void	joystick_mkmotion(const joystick_params_s *params,
                const joystick_mappings_s *mappings,
                const or_joystick_state *state, joystick_input_s *input);

#endif /* H_JOYSTICK_CODELS */
